/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.connector.jms.activemq.client;

import java.util.Properties;

import javax.jms.ConnectionFactory;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;

import org.apache.activemq.ra.ActiveMQManagedConnectionFactory;
import org.apache.activemq.ra.ActiveMQResourceAdapter;
import org.apache.commons.beanutils.BeanUtils;
import org.bluestemsoftware.specification.eoa.ext.connector.ConnectorException;
import org.bluestemsoftware.specification.eoa.ext.connector.jms.JMSConnector;
import org.bluestemsoftware.specification.eoa.ext.connector.jms.JMSConnectorFactory;
import org.bluestemsoftware.specification.eoa.system.container.ConnectionSupport;

public final class JMSConnectorFactoryImpl implements JMSConnectorFactory.Provider {

    private static final long serialVersionUID = 1L;
    
    public static String NAME = "activemq-client-connector";
    
    public JMSConnectorFactoryImpl() {
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.ExtensionFactory$Provider#spi_getName()
     */
    public String spi_getName() {
        return NAME;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.connector.ConnectorFactory$Provider#spi_getExtensionImpl()
     */
    public String spi_getConnectorImpl() {
        return JMSConnectorImpl.IMPL;
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.connector.jms.JMSConnectorFactory$Provider#spi_createConnector(java.lang.String, org.bluestemsoftware.specification.eoa.system.container.ConnectionSupport, java.util.Properties, boolean)
     */
    public JMSConnector.Provider spi_createConnector(String name, ConnectionSupport connectionSupport, Properties properties, boolean isXA) throws ConnectorException {

        ConnectionFactory connectionFactory;
        if (isXA) {
            connectionFactory = createXAConnectionFactory(properties);
        } else {
            connectionFactory = createConnectionFactory((ConnectionManager)connectionSupport, properties);
        }

        return new JMSConnectorImpl(connectionFactory);

    }

    private ConnectionFactory createConnectionFactory(ConnectionManager connectionManager, Properties properties) throws ConnectorException {

        ActiveMQResourceAdapter resourceAdapter = new ActiveMQResourceAdapter();
        if (properties != null) {
            try {
                BeanUtils.populate(resourceAdapter, properties);
            } catch (Throwable th) {
                throw new ConnectorException("Error setting properties on Connector of type '"
                        + JMSConnector.TYPE
                        + "' and impl '"
                        + JMSConnectorImpl.IMPL
                        + "'. "
                        + th);
            }
        }

        ConnectionFactory connectionFactory = null;
        try {
            ActiveMQManagedConnectionFactory managedConnectionFactory = new ActiveMQManagedConnectionFactory();
            managedConnectionFactory.setResourceAdapter(resourceAdapter);
            connectionFactory = (ConnectionFactory)managedConnectionFactory
                    .createConnectionFactory(connectionManager);
        } catch (ResourceException re) {
            throw new ConnectorException("Error creating connector. " + re);
        }

        return connectionFactory;

    }

    private ConnectionFactory createXAConnectionFactory(Properties properties) throws ConnectorException {

        // i'm not sure why active mq doesn't provide an ActiveMQManagedXAConnectionFactory.
        // we could use jenks to provide pooling support along with our tx manager, i.e. by
        // uncommenting code below, but jenks requires too much baggage, i.e. it adds alot of
        // the core geronimo artifacts, etc ... AND we would have to change our API to access
        // TM outside context of ConnectionManager. yuk

        /*
         * ActiveMQXAConnectionFactory xaConnectionFactory = new ActiveMQXAConnectionFactory();
         * 
         * if (properties != null) { for (Entry<Object, Object> entry : properties.entrySet()) {
         * String propertyName = (String)entry.getKey(); String propertyValue =
         * (String)entry.getValue(); try { PropertyUtils.setSimpleProperty(xaConnectionFactory,
         * propertyName, propertyValue); } catch (Throwable th) { throw new
         * ConnectorException("Error setting property '" + propertyName + "' on Connector of
         * type '" + REF + "' and impl '" + IMPL + "'. " + th); } } }
         * 
         * PooledSpringXAConnectionFactory pooledSpringConnectionFactory = new
         * PooledSpringXAConnectionFactory();
         * pooledSpringConnectionFactory.setConnectionFactory(xaConnectionFactory);
         * pooledSpringConnectionFactory.setTransactionManager(?);
         * 
         * return pooledSpringConnectionFactory;
         */

        throw new ConnectorException("Managed XA JMS ConnectionFactory not yet supported");

    }
  
}
